﻿using System.Collections.Generic;
using System.Drawing;

using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 라우팅 헬퍼
    /// </summary>
    public class RoutingHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 맵 푸시핀 리스트
        /// </summary>
        private List<MapPushpin> pushpinList = new List<MapPushpin>();

        /// <summary>
        /// 다각선
        /// </summary>
        private MapPolyline polyline = new MapPolyline()
        {
            StrokeWidth         = 4,
            SelectedStrokeWidth = 4,
            Stroke              = Color.FromArgb(0xff, 0xfe, 0x72, 0xff)
        };

        /// <summary>
        /// 경로점 리스트
        /// </summary>
        private List<RouteWaypoint> waypointList = new List<RouteWaypoint>();

        /// <summary>
        /// 현재 후자
        /// </summary>
        private char currentLatter = 'A';

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 푸시핀 리스트 - PushpinList

        /// <summary>
        /// 푸시핀 리스트
        /// </summary>
        public List<MapPushpin> PushpinList
        {
            get
            {
                return this.pushpinList;
            }
        }

        #endregion
        #region 경로점 리스트 - WaypointList

        /// <summary>
        /// 경로점 리스트
        /// </summary>
        public List<RouteWaypoint> WaypointList
        {
            get
            {
                return this.waypointList;
            }
        }

        #endregion
        #region 다각선 - Polyline

        /// <summary>
        /// 다각선
        /// </summary>
        public MapPolyline Polyline
        {
            get
            {
                return this.polyline;
            }
            set
            {
                this.polyline = value;
            }
        }

        #endregion
        #region 카운트 - Count

        /// <summary>
        /// 카운트
        /// </summary>
        public int Count
        {
            get
            {
                return this.pushpinList.Count;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 경로 구축하기 - BuildRoute(pointEnumerable)

        /// <summary>
        /// 경로 구축하기
        /// </summary>
        /// <param name="pointEnumerable">포인트 열거 가능 목록</param>
        public void BuildRoute(IEnumerable<GeoPoint> pointEnumerable)
        {
            this.polyline.Points.Clear();

            foreach(GeoPoint point in pointEnumerable)
            {
                this.polyline.Points.Add(point);
            }
        }

        #endregion
        #region 항목 추가하기 - AddItem(pushpin)

        /// <summary>
        /// 항목 추가하기
        /// </summary>
        /// <param name="pushpin">푸시핀</param>
        public void AddItem(MapPushpin pushpin)
        {
            this.pushpinList.Add(pushpin);

            this.waypointList.Add
            (
                new RouteWaypoint
                (
                    ((LocationInformation)pushpin.Information).DisplayName,
                    (GeoPoint)pushpin.Location
                )
            );

            pushpin.Text = (this.currentLatter++).ToString();
        }

        #endregion
        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            this.polyline.Points.Clear();

            this.pushpinList.Clear();

            this.waypointList.Clear();

            this.currentLatter = 'A';
        }

        #endregion
    }
}